@@ -1,4 +1,5 @@
 import base64
+import binascii
 import email
 import email.utils
 import logging
@@ -7,15 +8,20 @@ import quopri
 import re
 import tempfile
 from base64 import b64decode
-from email import message_from_string
+from email import errors, message_from_string
+from email.header import decode_header, make_header
+from email.message import Message
 from email.parser import HeaderParser
 from email.utils import getaddresses
 
 from parse_emails.common import convert_to_unicode
 from parse_emails.handle_msg import handle_msg
 
+logger = logging.getLogger('parse_emails')
+
 MIME_ENCODED_WORD = re.compile(r'(.*)=\?(.+)\?([B|Q])\?(.+)\?=(.*)')  # guardrails-disable-line
 ENCODINGS_TYPES = {'utf-8', 'iso8859-1'}
+headerRE = re.compile(r'^(From |[\041-\071\073-\176]*:|[\t ])')
 
 
 def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, max_depth=3, bom=False, original_depth=3):
@@ -36,6 +42,10 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
         if isinstance(file_data, bytes):
             file_data = file_data.decode('utf-8', 'ignore')
 
+        logger.debug(f'Before check_if_file_starts_with_header, {file_data=}')
+        file_data = check_if_file_starts_with_header(file_data)
+        logger.debug(f'After check_if_file_starts_with_header, {file_data=}')
+
         parser = HeaderParser()
         headers = parser.parsestr(file_data)
 
@@ -74,8 +84,14 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
             else:
                 headers_map[item[0]] = value
 
-        eml = message_from_string(file_data)
+        eml = None
+        try:
+            eml = create_message_from_string(file_data)
+        except Exception as e:
+            logger.info(f'Exception calling create_message_from_string, {e}, from {file_data=}')
+
         if not eml:
+            logger.info('Empty eml after create_message_from_string')
             raise Exception("Could not parse eml file!")
 
         if parse_only_headers:
@@ -94,11 +110,16 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
 
         while parts:
             part = parts.pop()
+
+            logger.debug(f'Iterating over parts. Current part: {part.get_content_type()=}')
             if (part.is_multipart() or part.get_content_type().startswith('multipart')) \
                     and "attachment" not in part.get("Content-Disposition", ""):
                 parts += [part_ for part_ in part.get_payload() if isinstance(part_, email.message.Message)]
 
-            elif part.get_filename() or "attachment" in part.get("Content-Disposition", ""):
+            elif part.get_filename()\
+                    or "attachment" in part.get("Content-Disposition", "")\
+                    or part.get("X-Attachment-Id")\
+                    or ("image" in part.get("Content-Type", '') and part.get("Content-Transfer-Encoding") == "base64"):
 
                 attachment_content_id = part.get('Content-ID')
                 attachment_content_disposition = part.get('Content-Disposition')
@@ -132,11 +153,13 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
 
                             except TypeError:
                                 pass  # In case the file is a string, decode=True for get_payload is not working
+                            except binascii.Error:
+                                pass
 
                     elif isinstance(part.get_payload(), str):
                         file_content = part.get_payload(decode=True)
                     else:
-                        logging.debug("found eml attachment with Content-Type=message/rfc822 but has no payload")
+                        logger.debug("found eml attachment with Content-Type=message/rfc822 but has no payload")
 
                     if file_content:
                         # save the eml to war room as file entry
@@ -176,13 +199,16 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
                             attachment_file_name = individual_message.get_filename()
                             attachment_content_id = individual_message.get('Content-ID')
                             attachment_content_disposition = individual_message.get('Content-Disposition')
-                            if attachment_file_name is None:
-                                attachment_file_name = f"unknown_file_name{i}"
+                            if not attachment_file_name and not attachment_content_ids and 'text/html' in individual_message.get_content_type():
+                                html = decode_content(individual_message)
+                            else:
+                                if attachment_file_name is None:
+                                    attachment_file_name = f"unknown_file_name{i}"
 
-                            attachment_content.append(msg_info)
-                            attachment_names.append(attachment_file_name)
-                            attachment_content_ids.append(attachment_content_id)
-                            attachment_content_dispositions.append(attachment_content_disposition)
+                                attachment_content.append(msg_info)
+                                attachment_names.append(attachment_file_name)
+                                attachment_content_ids.append(attachment_content_id)
+                                attachment_content_dispositions.append(attachment_content_disposition)
                     else:
                         file_content = part.get_payload(decode=True)
                         if attachment_file_name.endswith('.p7s') or not file_content:
@@ -229,20 +255,22 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
 
             elif part.get_content_type() == 'text/plain':
                 text = decode_content(part)
-
-        if attachments_images and html:  # embed images into the HTML body.
-            html = embed_images_to_html_body(html=html, attachments_images=attachments_images)
+            else:
+                logger.info(f'Not handling part of type {part.get_content_type()=}')
 
         email_data = None
         # if we are parsing a signed attachment there can be one of two options:
-        # 1. it is 'multipart/signed' so it is probably a wrapper and we can ignore the outer "email"
-        # 2. if it is 'multipart/signed' but has 'to' address so it is actually a real mail.
+        # 1. it is 'multipart/signed' so it is probably a wrapper, and we can ignore the outer "email"
+        #    However, we should save its AttachmentsData.
+        # 2. if it is 'multipart/signed' but has 'to'/'from'/'subject' fields, so it is actually a real mail.
         if 'multipart/signed' not in eml.get_content_type() \
-                or ('multipart/signed' in eml.get_content_type() and
-                    (extract_address_eml(eml, 'to') or extract_address_eml(eml, 'from') or eml.get('subject'))):
+            or ('multipart/signed' in eml.get_content_type() and
+                ((extract_address_eml(eml, 'to') or extract_address_eml(eml, 'from') or eml.get('subject')) or
+                 attachment_names)):
             email_data = {
                 'To': extract_address_eml(eml, 'to'),
                 'CC': extract_address_eml(eml, 'cc'),
+                'BCC': extract_address_eml(eml, 'bcc'),
                 'From': extract_address_eml(eml, 'from'),
                 'Subject': convert_to_unicode(unfold(eml['Subject'])),
                 'HTML': convert_to_unicode(html, is_msg_header=False),
@@ -266,34 +294,56 @@ def handle_eml(file_path, b64=False, file_name=None, parse_only_headers=False, m
         return email_data, attached_emails
 
 
-def embed_images_to_html_body(html, attachments_images):
+def create_message_from_string(file_data: str) -> Message:
     """
-    Embed images into the HTML body by changing the src of the image to the image content in base64
-
+    Parse a string into a Message object model.
+    and checks if there is a multipart error we try to fix it
     Args:
-        html (str): the HTML of the email.
-        attachments_images (List(tuple[str, str])): a list of tuples containing attachment IDs and the image content in
-            base64.
-
-    https://sendgrid.com/blog/embedding-images-emails-facts/
-
+        file_data (str) : the email data as string.
     Returns:
-        str: the HTML embedded with images.
+        the eml parse obj (Message)
     """
-    if '<img' not in html:
-        return html
+    eml = message_from_string(file_data)
+    if eml.defects:
+        eml = handle_multi_part_error(eml)
+    return eml
 
-    for attachment_id, image_base64 in attachments_images:
-        if attachment_id:  # in p7m files types we can have png files without an attachment ID.
-            attachment_id = re.sub('<|>', '', attachment_id)  # remove < and > from the attachment-ID.
-            # '<image001.jpg@01D8B147.CFCD4400>' --> image001.jpg@01D8B147.CFCD4400
-            image_base64 = re.sub('\n|\r', '', image_base64)  # remove escaping chars
-            attachment_cid_pattern = f'src="cid:{attachment_id}"'
-            if attachment_cid_pattern in html:
-                html = html.replace(
-                    attachment_cid_pattern, f'src="data:image/jpeg;base64,{image_base64}"'
-                )
-    return html
+
+def handle_multi_part_error(eml: Message):
+    """
+    This function handles a multipart which is missing boundary,
+    and checks if the boundary exists only once in the file, if so it removes it.
+    Args:
+        the eml parse obj (Message) : the email data.
+    Returns:
+        the eml parse obj (Message)
+    """
+    logger.debug('handle_multi_part_error')
+    for defect in eml.defects:
+        if isinstance(defect, errors.MultipartInvariantViolationDefect):
+            boundary = eml.get_boundary()
+            file_data = eml.as_string()
+            if file_data.count(boundary) == 1:
+                param = eml.get("Content-Type").replace('\r\n', '\n')
+                file_data = file_data.replace(f'Content-Type: {param}\n', '')
+                eml = message_from_string(file_data)
+            break
+    return eml
+
+
+def check_if_file_starts_with_header(file_data: str):
+    """
+    This function checks if the file data starts with headers, and if not it deletes the lines before the headers.
+    Args:
+        file_data (str) : the email data.
+    Returns:
+        file_data (str) : the email data without text before the headers.
+    """
+    for idx, line in enumerate(file_data.splitlines()):
+        if headerRE.match(line):
+            file_data = file_data.split("\n", idx)[idx]
+            break
+    return file_data
 
 
 def unfold(s):
@@ -315,6 +365,9 @@ def decode_content(mime):
       Decode content
     """
     charset = mime.get_content_charset()
+    content_transfer_encoding = mime.get('content-transfer-encoding', '').rstrip(' ')
+    if (charset == 'utf-8' and content_transfer_encoding == '8bit'):
+        return mime.get_payload()
     payload = mime.get_payload(decode=True)
     try:
         if payload:
@@ -323,17 +376,26 @@ def decode_content(mime):
             elif charset == 'iso-8859-2':
                 return payload.decode('iso-8859-2')
             elif charset == 'utf-8':
-                return payload.decode('utf-8')
+                return payload.decode('utf-8', errors='ignore')
             elif charset in ('gb2312', 'gb18030'):  # chinese encodings
                 return payload.decode('gb18030')
+            elif charset == 'iso-2022-jp':
+                return payload.decode('iso-2022-jp')
+            elif charset == 'big5':
+                return payload.decode('big5')
+            elif charset == 'gbk':
+                return payload.decode('gbk')
             else:
                 return payload.decode("raw-unicode-escape")
         else:
+            logger.debug('decode_content, empty payload, returning an empty string.')
             return ''
 
-    except UnicodeDecodeError:
+    except UnicodeDecodeError as ude:
+        logger.info(f'Exception trying to decode content: {ude}')
         payload = mime.get_payload()
         if isinstance(payload, str):
+            logger.info(f'Exception trying to decode content. payload is str, returning it. {ude}')
             return payload
 
 
@@ -361,26 +423,58 @@ def mime_decode(word_mime_encoded):
     return prefix + byte_string.decode(charset) + suffix
 
 
-def get_email_address(eml, entry):
+def get_email_address(eml, header_name):
     """
-    This function gets email addresses from an eml object, i.e eml[entry].
+    This function gets email addresses from an eml object, i.e eml[header_name].
     Args:
-        eml : Email object.
-        entry (str) : entry to look for in the email. i.e ('To', 'CC', 'From')
+        eml: Email object.
+        header_name (str): header to look for in the email. i.e ('To', 'CC', 'From')
     Returns:
-        res (str) : string of all required email addresses.
+        res (str): string of all required email addresses.
     """
-    if entry == 'from':
-        gel_all_values_from_email_by_entry = [str(current_eml_no_newline).replace('\r\n', '').replace('\n', '')
-                                              for current_eml_no_newline in eml.get_all(entry, [])]
+    if header_name in ['from']:
+        header_value = [str(current_eml_no_newline).replace('\r\n', '').
+                        replace('\n', '').replace('\t', '')
+                        for current_eml_no_newline in eml.get_all(header_name, [])]
     else:
-        gel_all_values_from_email_by_entry = eml.get_all(entry, [])
-    addresses = getaddresses(gel_all_values_from_email_by_entry)
-    if addresses:
-        res = [item[1] for item in addresses]
-        res = ', '.join(res)
-        return res
-    return ''
+        header_value = eml.get_all(header_name, [])
+
+    res = []
+    addresses = header_value[0].split(',') \
+        if header_value else []
+    for address in addresses:
+        if 'unknown-8bit' in address:
+            try:
+                decoded = str(make_header(decode_header(address)))
+                result = get_address(decoded)
+            except UnicodeDecodeError:
+                email_address = get_address(address)
+                result = str(make_header(decode_header(email_address)))
+        else:
+            result = get_address(address)
+
+        if result:
+            res.append(result)
+
+    return ', '.join(res)
+
+
+def get_address(email_address):
+    """
+    Extracts valid email addresses from a string using email.utils.getaddresses.
+
+    Args:
+        email_address (str): A string containing one or more email addresses.
+
+    Returns:
+        str: A comma-separated string of valid email addresses.
+    """
+    try:
+        addresses = getaddresses([email_address], strict=False)
+    except TypeError:
+        addresses = getaddresses([email_address])
+    valid_addresses = [addr for _, addr in addresses if "@" in addr]
+    return ', '.join(valid_addresses)
 
 
 def extract_address_eml(eml, entry):
@@ -402,15 +496,32 @@ def extract_address_eml(eml, entry):
 
 def get_attachment_filename(part):
     attachment_file_name = None
-    if part.get_filename():
-        attachment_file_name = part.get_filename()
-
+    filename = part.get_filename()
+    if filename:
+        try:
+            decoded_header = decode_header(filename)
+            logger.debug(f"Decoded header for attachment: {decoded_header}")
+            attachment_file_name = str(make_header(decoded_header))
+        except LookupError:
+            if 'windows-874' in filename:
+                # If the file is encoded in windows-874 and contains the encoding
+                filename = filename.replace('windows-874', 'iso-8859-11')
+                attachment_file_name = str(make_header(decode_header(filename)))
+        except UnicodeDecodeError:
+            # fallback: ignore bad bits so we get the rest of the name
+            attachment_file_name = ''.join(
+                part.decode(charset or 'utf-8', errors='ignore')
+                for part, charset in decode_header(filename)
+            )
+            logger.debug(f"Failed to decode attachment {filename}, will ignore bad bits.")
     elif attachment_file_name is None and part.get('filename'):
         attachment_file_name = os.path.normpath(part.get('filename'))
         if os.path.isabs(attachment_file_name):
             attachment_file_name = os.path.basename(attachment_file_name)
     else:
-        if not isinstance(part.get_payload(), list):
+        if attach_id := part.get("X-Attachment-Id"):
+            attachment_file_name = attach_id
+        elif not isinstance(part.get_payload(), list):
             attachment_file_name = 'unknown_file_name'
         else:
             for payload in part.get_payload():
@@ -429,7 +540,7 @@ def decode_attachment_payload(message):
         # In some cases the body content is empty and cannot be decoded.
         msg_info = base64.b64decode(msg)
     except Exception as e:
-        logging.debug(f'exception while trying to decode_attachment_payload - {str(e)}')
+        logger.debug(f'exception while trying to decode_attachment_payload - {str(e)}')
         msg_info = str(msg)
     return msg_info
 
