@@ -52,7 +52,10 @@ from parse_emails.common import convert_to_unicode
 from parse_emails.constants import (DEFAULT_ENCODING, PROPS_ID_MAP,
                                     REGEX_EMAIL, USER_ENCODING)
 
+logger = logging.getLogger('parse_emails')
+
 MIME_ENCODED_WORD = re.compile(r'(.*)=\?(.+)\?([B|Q])\?(.+)\?=(.*)')  # guardrails-disable-line
+emailRegex = r'''(?i)(?:[a-z0-9!#$%&'*+/=?^_\x60{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_\x60{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])'''  # noqa: E501
 
 DATA_TYPE_MAP = {
     "0x0000": "PtypUnspecified",
@@ -90,11 +93,12 @@ DATA_TYPE_MAP = {
 }
 
 EMBEDDED_MSG_HEADER_SIZE = 24
+UTF_16_ENCODING = True
 
 
 def handle_msg(file_path, file_name, parse_only_headers=False, max_depth=3, original_depth=3):
     if max_depth == 0:
-        return None, []
+        return None, [], []
 
     msg = MsOxMessage(file_path)
     if not msg:
@@ -105,7 +109,7 @@ def handle_msg(file_path, file_name, parse_only_headers=False, max_depth=3, orig
     headers, headers_map = create_headers_map(msg_dict.get('Headers'))
 
     if parse_only_headers:
-        return {"HeadersMap": headers_map}, []
+        return {"HeadersMap": headers_map}, [], []
 
     eml_attachments, attachments_data = save_attachments(msg.get_all_attachments(), max_depth - 1)
     # add eml attached emails
@@ -188,7 +192,7 @@ def get_msg_mail_format(msg_dict):
     try:
         return msg_dict.get('Headers', 'Content-type:').split('Content-type:')[1].split(';')[0]
     except Exception as e:
-        logging.debug(f'Got exception while trying to get msg mail format - {str(e)}')
+        logger.debug(f'Got exception while trying to get msg mail format - {str(e)}')
         return ''
 
 
@@ -342,7 +346,7 @@ class DataModel:
         if data_value:
             try:
                 if USER_ENCODING:
-                    logging.debug(f'Using argument user_encoding: {USER_ENCODING} to decode parsed message.')
+                    logger.debug(f'Using argument user_encoding: {USER_ENCODING} to decode parsed message.')
                     return data_value.decode(USER_ENCODING, errors="ignore")
                 res = chardet.detect(data_value)
                 enc = res['encoding'] or 'ascii'  # in rare cases chardet fails to detect and return None as encoding
@@ -350,17 +354,17 @@ class DataModel:
                     if enc.lower() == 'windows-1252' and res['confidence'] < 0.9:
 
                         enc = DEFAULT_ENCODING if DEFAULT_ENCODING else 'windows-1250'
-                        logging.debug('Encoding detection confidence below threshold {}, '
-                                      'switching encoding to "{}"'.format(res, enc))
+                        logger.debug('Encoding detection confidence below threshold {}, '
+                                     'switching encoding to "{}"'.format(res, enc))
 
                     temp = data_value
                     data_value = temp.decode(enc, errors='ignore')
                     if '\x00' in data_value:
-                        logging.debug('None bytes found on encoded string, will try use utf-16-le '
-                                      'encoding instead')
+                        logger.debug('None bytes found on encoded string, will try use utf-16-le '
+                                     'encoding instead')
                         data_value = temp.decode("utf-16-le", errors="ignore")
 
-                elif b'\x00' not in data_value:
+                elif b'\x00' not in data_value or not UTF_16_ENCODING:
                     data_value = data_value.decode("ascii", errors="ignore")
                 else:
                     data_value = data_value.decode("utf-16-le", errors="ignore")
@@ -478,16 +482,6 @@ class Message:
         self._set_properties()
         self._set_attachments()
         self._set_recipients()
-        self._embed_images_to_html_body()
-
-    def _embed_images_to_html_body(self):
-        # embed images into html body
-        if self.attachments and self.html:
-            for attachment in self.attachments:
-                if attachment.AttachContentId and f'src="cid:{attachment.AttachContentId}"' in self.html:
-                    img_base64 = base64.b64encode(attachment.data).decode('ascii')
-                    self.html = self.html.replace(f'src="cid:{attachment.AttachContentId}"',
-                                                  f'src="data:image/png;base64, {img_base64}"')
 
     def _get_attachments_names(self):
         names = []
@@ -712,6 +706,7 @@ class Message:
         return attachment_entries
 
     def _get_property_data(self, directory_name, directory_entry, is_list=False):
+        global UTF_16_ENCODING
         directory_entry_name = directory_entry.name
         if is_list:
             stream_name = [directory_name, directory_entry_name]
@@ -729,7 +724,7 @@ class Message:
         property_name = property_details.get("name")
         property_type = property_details.get("data_type")
         if not property_type:
-            logging.debug(f'could not parse property type, skipping property "{property_details}"')
+            logger.debug(f'could not parse property type, skipping property "{property_details}"')
             return None
 
         try:
@@ -737,11 +732,15 @@ class Message:
         except OSError:
             raw_content = ''
         if not raw_content:
-            logging.debug('Could not read raw content from stream "{}", '
-                          'skipping property "{}"'.format(stream_name, property_details))
+            logger.debug('Could not read raw content from stream "{}", '
+                         'skipping property "{}"'.format(stream_name, property_details))
             return None
 
+        if directory_entry_name.endswith('001E'):
+            # UTF_16_ENCODING is used specifically in PtypString().
+            UTF_16_ENCODING = False
         property_value = self._data_model.get_value(raw_content, data_type=property_type)
+        UTF_16_ENCODING = True
         if property_value:
             property_detail = {property_name: property_value}
         else:
@@ -838,12 +837,30 @@ class Message:
                 rtf_body = compressed_rtf.decompress(compressed_rtf_body)
                 # Need to remove the following row and the _simplify_text_for_rtf_parsing() func when version 0.0.3 of
                 # the RTFDE.deencapsulate package is released.
-                rtf_body = self._simplify_text_for_rtf_parsing(rtf_body)
-                from RTFDE.deencapsulate import DeEncapsulator
-                rtf_obj = DeEncapsulator(rtf_body)
-                rtf_obj.deencapsulate()
-                if rtf_obj.content_type == 'html':
-                    self.html = rtf_obj.html
+                try:
+                    rtf_body = self._simplify_text_for_rtf_parsing(rtf_body)
+                    from RTFDE.deencapsulate import DeEncapsulator
+                    rtf_obj = DeEncapsulator(rtf_body)
+                    rtf_obj.deencapsulate()
+                    if rtf_obj.content_type == 'html':
+                        self.html = rtf_obj.html
+                except Exception as e:
+                    # in case of decoding issues from DeEncapsulator.deencapsulate please refer to https://github.com/seamustuohy/RTFDE/issues/33
+                    logger.debug(f'Got exception while trying to get html from rtf using RTFDE lib - {str(e)}')
+                    try:
+                        logger.debug('Trying to deencapsulate using pandoc, please see the source code for more details')
+
+                        # need to install the pandoc executable
+                        import subprocess
+                        subprocess.run(['apk', 'add', 'pandoc'], check=True, capture_output=True)
+
+                        import pypandoc
+                        html_content = pypandoc.convert_text(rtf_body, 'html', format='rtf')
+                        if html_content:
+                            self.html = html_content
+                    except Exception as e:
+                        self.html = "could not parse RTF due to known issue with special characters"
+                        logger.debug(f'Ignore to get html from rtf using pypandoc lib - {str(e)}')
 
     def _set_recipients(self):
         recipients = self.recipients
@@ -1046,10 +1063,28 @@ def parse_email_headers(header, raw=False):
         "BCC": [],
         "Reply-To": [],
     }
-
+    format_issue = False
     for addr in email_address_headers.keys():
-        for (name, email_address) in email.utils.getaddresses(headers.get_all(addr, [])):
-            email_address_headers[addr].append(f"{name} <{email_address}>")
+        address_header = headers.get_all(addr, [])
+        email_addresses = email.utils.getaddresses(address_header)
+
+        # This workaround addresses an issue with email addresses that have a comma in the display name
+        # but are not enclosed in quotation marks (XSUP-41796). In such cases, the email.utils.getaddresses function
+        # fails to parse the address correctly, as it expects display names with special characters
+        # (like commas) to be wrapped in quotes, according to RFC 5322.
+        if addr == 'From' and len(email_addresses) > 1:
+            for (name, email_address) in email_addresses:
+                # If email.utils.getaddresses returns a list, we check the email addresses returned.
+                # If any of them contain invalid email addresses, we remove them and keep only the valid ones.
+                if not re.findall(emailRegex, email_address):
+                    format_issue = True
+                    email_addresses.remove((name, email_address))
+
+        for (name, email_address) in email_addresses:
+            if format_issue:
+                email_address_headers[addr].append(f"<{email_address}>")
+            else:
+                email_address_headers[addr].append(f"{name} <{email_address}>")
 
     parsed_headers = dict(headers)
     parsed_headers.update(email_address_headers)
